<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use  Util\DevOpsEnvironment AS DevOpsEnvironment;
use Util\InstanceProvider;


class CpWebService {

    protected $entityManager;

    /** @var DevOpsEnvironment */
    private $devOpsEnvironment;

    /** @var \Util\Utilities */
    private $environment;

    /** @var \Teknisa\Model\Teknisa\Repositories\Linpluso $useLicenseRepository */
    protected  $useLicenseRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Pais $countryRepository */
    protected  $countryRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Estado $stateRepository */
    protected  $stateRepository ;
    /** @var \Teknisa\Model\Geral\Repositories\Municipio $stateRepository */
    protected  $municipioRepository ;
    /** @var \Teknisa\Model\Teknisa\Repositories\Licencauso $licencausoRepository */
    protected  $licencausoRepository;
     /** @var \Factory\CustomizationFiles $productCallService */
     protected $customizationFilesFactory;

    public function __construct(EntityManager $entityManager, \Zeedhi\DevOps\Util\Utilities $devOpsEnvironment, \Util\Utilities $environment,
                                \Factory\CustomizationFiles $customizationFilesFactory) {
        $this->entityManager = $entityManager;
        $this->useLicenseRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Linpluso');
        $this->devOpsEnvironment = $devOpsEnvironment;
        $this->countryRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Pais');
        $this->stateRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Estado');
        $this->municipioRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Municipio');
        $this->licencausoRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Licencauso');
        $this->customizationFilesFactory = $customizationFilesFactory;
        $this->environment = $environment;
    }

    public function findUseLicenseByCdcliente($params, $nrorg) {
        $cdCliente = $this->licencausoRepository->findCdclienteByNrorg($nrorg)[0]['CDCLIENTE'];
        $filter = $this->devOpsEnvironment->createQueryFiltered($params);
        return $this->useLicenseRepository->getUseLicense($cdCliente, $filter);
    }

    public function findCity($sgEstadoFilter, $countryIdFilter,$filters) {
        return  InstanceProvider::getRepository()->getLocalMunicipio()->ApiGetCity($sgEstadoFilter, $countryIdFilter, $filters);
    }

    public function findState($countryIdFilter, $filters) {
        return InstanceProvider::getRepository()->getLocalEstado()->ApiGetStateByCountry($countryIdFilter, $filters);
    }

    public function validateCustomizationFiles($nrinsjurluso, $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $idtpijurluso)
    {
        // Validação do numero de inscrição parametrizado pelo tipo de organização
        // Caso F valida mascara CPF, caso J Mascara CNPJ
        if($idtpijurluso == 'F') {
            if (!preg_match("/^\d{3}\.\d{3}\.\d{3}\-\d{2}$/", $nrinsjurluso)) {
                throw new \Exception('CPF inválido. Envie o valor no formato: XXX.XXX.XXX-XX');
            }
        } else if ($idtpijurluso == 'J') {
            if (!preg_match("/^\d{2}\.\d{3}\.\d{3}\/\d{4}\-\d{2}$/", $nrinsjurluso)) {
                throw new \Exception('CNJP inválido. Envie o valor no formato: XX.XXX.XXX/XXXX-XX');
            }
        }

        // Valida formato do CEP
        if (!preg_match("/^[0-9]{5}-[0-9]{3}$/", $nrcepluso)) {
            throw new \Exception('CEP inválido. Envie o valor no formato: XXXXX-XXX');
        }

        // Valida codigos de pais, estado, municipio
        $filters = " AND CDPAIS = " . $cdpais;
        $country = $this->countryRepository->obtemTodosPaises($filters);
        if(empty($country)){
            throw new \Exception('País não encontrado.');
        }

        $filters = "AND E.SGESTADO = '" .$sgestado . "'";
        $state = $this->stateRepository->getStateByCountry($cdpais, $filters);
        if(empty($state)){
            throw new \Exception('Estado não encontrado.');
        }

        $filters = "AND CDMUNICIPIO = '" . $cdmunicipio . "'";
        $city = $this->municipioRepository->obtemPorSiglaEstado($sgestado, $filters);
        if(empty($city)){
            throw new \Exception('Município não encontrado.');
        }
    }

    public function removeCustomizationFile($licencauso, $nrorg, $active) {
        $connection = $this->entityManager->getConnection();
        try {
            $cdCliente = $this->licencausoRepository->findCdclienteByNrorg($nrorg)[0]['CDCLIENTE'];
            $connection->beginTransaction();
            /** @var  \Teknisa\Model\Teknisa\Entities\Licencauso $customizationFile*/
            $customizationFile = $this->licencausoRepository->findOneBy(array('cdcliente' => $cdCliente, 'nrlicencauso' => $licencauso));
            $customizationFile->setIdlusoativ($active);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveCustomizationFiles($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                                           $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                                           $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                                           $cdlinpclie, $nrlicencauso, $idmodproc, $idlocaproc, $isNew, $idlusoativ, $idtpijurluso, $nrorg) {
        $connection = $this->entityManager->getConnection();
        try {
            $cdcliente = $this->licencausoRepository->findCdclienteByNrorg($nrorg)[0]['CDCLIENTE'];
            $this->entityManager->beginTransaction();
            /** @var  \Teknisa\Model\Teknisa\Entities\Licencauso $customizationFile*/
            $customizationFile = $this->licencausoRepository->findOneBy(array('cdcliente' => $cdcliente, 'cdlocaluso' => $cdlocaluso));
            if($isNew) {
                if (!$customizationFile) {
                    $customizationFile = $this->customizationFilesFactory->saveLicencauso($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                        $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                        $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                        $cdcliente, $nrlicencauso, $idmodproc, $idlocaproc, false, 'S', 1, $idtpijurluso);
                } else {
                    $idAtivo = $customizationFile->getIdlusoativ() == 'S' ? 'Ativo' : 'Inativo';
                    throw new \Exception('Inclusão bloqueada! <br/> Existe uma Licença de Uso já cadastrada com o código "'.$cdlocaluso.'" <br/> ('.$customizationFile->getNmfantluso().' - '.$idAtivo.')');
                }
            } else {
                $customizationFile = $this->customizationFilesFactory->saveLicencauso($cdlocaluso, $nmfantluso, $nmrazsocluso, $nrinsjurluso,
                    $nrinsestluso, $nrinsmuniluso, $nmcontluso, $nmdeptoluso, $idfaturafilisep, $dsendeluso, $nmbairluso,
                    $nrcepluso, $cdpais, $sgestado, $cdmunicipio, $nrteleluso, $nrramalluso, $nrfaxluso, $dsemailcomlu,
                    $cdcliente, $nrlicencauso, $idmodproc, $idlocaproc, $customizationFile, $idlusoativ, 1, $idtpijurluso);
            }
            $productLineRelatedToUseLicenses = $this->useLicenseRepository->findBy(array('cdcliente' => $cdcliente, 'nrlicencauso' => $nrlicencauso));
            $productLineRelatedToUseLicenseIdArray = array();
            foreach($productLineRelatedToUseLicenses as $productLineRelatedToUseLicense){
                if(!in_array($productLineRelatedToUseLicense->getCdlinpclie(), $cdlinpclie)){
                    $this->entityManager->remove($productLineRelatedToUseLicense);
                }else{
                    array_push($productLineRelatedToUseLicenseIdArray, $productLineRelatedToUseLicense->getCdlinpclie());
                }
            }
            if($cdlinpclie){
                foreach($cdlinpclie as $userLicenseId) {
                    if (!in_array($userLicenseId, $productLineRelatedToUseLicenseIdArray)) {
                        $linpluso = $this->customizationFilesFactory->createLinpluso($cdcliente, $nrlicencauso, $userLicenseId);
                        $this->entityManager->persist($linpluso);
                    }
                }
            }


            $this->entityManager->persist($customizationFile);
            $this->entityManager->flush();
            $connection->commit();
            return array('CDLOCALUSO' => $customizationFile->getCdlocaluso());
        } catch(\Exception $e) {
            $connection->rollBack();

            if(strpos($e->getMessage(), 'ORA-2'))
            {
                $message = explode('ORA-' , $e->getMessage())[1];
                $message = explode(':' , $message)[1];
                throw new \Exception($message);
            }
            throw $e;
        }
    }

}
